@@ -1,8 +1,8 @@
 from pccm import builder, core, middlewares
 from pccm.core import Argument, Class, FunctionCode, ParameterizedClass
 from pccm.core.markers import (constructor, destructor, external_function,
-                               member_function, static_function)
-from pccm.middlewares import pybind
+                               member_function, skip_inherit, static_function)
+from pccm.middlewares import expose_main, pybind
 from pccm.targets import cuda
 
 
