@@ -1,7 +1,15 @@
-from typing import List, Optional
+import contextlib
+import enum
+from typing import Dict, List, Optional, Set, Tuple, Union
 
-from pccm.core import (ConstructorMeta, DestructorMeta, ExternalFunctionMeta,
-                       MemberFunctionMeta, StaticMemberFunctionMeta, markers)
+from ccimport import compat
+
+from pccm.core import (Argument, ConstructorMeta, DestructorMeta,
+                       ExternalFunctionMeta, FunctionCode, MemberFunctionMeta,
+                       StaticMemberFunctionMeta, markers)
+
+if compat.Python3_6AndLater:
+    from .cuda_ptx import CacheOpLd, CacheOpSt, PTXCode, PTXContext, RegDType
 
 
 class CudaMemberFunctionMeta(MemberFunctionMeta):
@@ -35,10 +43,15 @@ def cuda_global_function(func=None,
                          macro_guard: Optional[str] = None,
                          impl_loc: str = "",
                          impl_file_suffix: str = ".cu",
+                         launch_bounds: Optional[Tuple[int, int]] = None,
+                         header_only: Optional[bool] = None,
                          name=None):
     if attrs is None:
         attrs = []
     cuda_global_attrs = attrs + ["__global__"]
+    if launch_bounds is not None:
+        cuda_global_attrs.append("__launch_bounds__({}, {})".format(
+            launch_bounds[0], launch_bounds[1]))
     return markers.external_function(func,
                                      name=name,
                                      inline=inline,
@@ -46,7 +59,8 @@ def cuda_global_function(func=None,
                                      macro_guard=macro_guard,
                                      impl_loc=impl_loc,
                                      impl_file_suffix=impl_file_suffix,
-                                     attrs=cuda_global_attrs)
+                                     attrs=cuda_global_attrs,
+                                     header_only=header_only)
 
 
 def member_function(func=None,
@@ -60,6 +74,7 @@ def member_function(func=None,
                     macro_guard: Optional[str] = None,
                     impl_loc: str = "",
                     impl_file_suffix: str = ".cu",
+                    header_only: Optional[bool] = None,
                     name=None):
     if forceinline or inline:
         assert forceinline is not inline, "can't set both inline and forceinline"
@@ -83,6 +98,7 @@ def member_function(func=None,
                                   macro_guard=macro_guard,
                                   impl_loc=impl_loc,
                                   impl_file_suffix=impl_file_suffix,
+                                  header_only=header_only,
                                   attrs=attrs)
     return markers.meta_decorator(func, meta)
 
@@ -97,6 +113,7 @@ def static_function(func=None,
                     macro_guard: Optional[str] = None,
                     impl_loc: str = "",
                     impl_file_suffix: str = ".cu",
+                    header_only: Optional[bool] = None,
                     name=None):
     if forceinline or inline:
         assert forceinline is not inline, "can't set both inline and forceinline"
@@ -110,15 +127,14 @@ def static_function(func=None,
     if attrs is None:
         attrs = []
     attrs.extend(cuda_global_attrs)
-    meta = CudaStaticMemberFunctionMeta(
-        name=name,
-        inline=inline,
-        constexpr=constexpr,
-        attrs=attrs,
-        macro_guard=macro_guard,
-        impl_loc=impl_loc,
-        impl_file_suffix=impl_file_suffix,
-    )
+    meta = CudaStaticMemberFunctionMeta(name=name,
+                                        inline=inline,
+                                        constexpr=constexpr,
+                                        attrs=attrs,
+                                        macro_guard=macro_guard,
+                                        impl_loc=impl_loc,
+                                        impl_file_suffix=impl_file_suffix,
+                                        header_only=header_only)
     return markers.meta_decorator(func, meta)
 
 
@@ -132,6 +148,7 @@ def external_function(func=None,
                       macro_guard: Optional[str] = None,
                       impl_loc: str = "",
                       impl_file_suffix: str = ".cu",
+                      header_only: Optional[bool] = None,
                       name=None):
     if forceinline or inline:
         assert forceinline is not inline, "can't set both inline and forceinline"
@@ -145,15 +162,14 @@ def external_function(func=None,
     if attrs is None:
         attrs = []
     attrs.extend(cuda_global_attrs)
-    meta = CudaExternalFunctionMeta(
-        name=name,
-        inline=inline,
-        constexpr=constexpr,
-        attrs=attrs,
-        macro_guard=macro_guard,
-        impl_loc=impl_loc,
-        impl_file_suffix=impl_file_suffix,
-    )
+    meta = CudaExternalFunctionMeta(name=name,
+                                    inline=inline,
+                                    constexpr=constexpr,
+                                    attrs=attrs,
+                                    macro_guard=macro_guard,
+                                    impl_loc=impl_loc,
+                                    impl_file_suffix=impl_file_suffix,
+                                    header_only=header_only)
     return markers.meta_decorator(func, meta)
 
 
@@ -167,6 +183,7 @@ def constructor(func=None,
                 macro_guard: Optional[str] = None,
                 impl_loc: str = "",
                 impl_file_suffix: str = ".cu",
+                header_only: Optional[bool] = None,
                 name=None):
     if forceinline or inline:
         assert forceinline is not inline, "can't set both inline and forceinline"
@@ -180,12 +197,11 @@ def constructor(func=None,
     if attrs is None:
         attrs = []
     attrs.extend(cuda_global_attrs)
-    meta = CudaConstructorMeta(
-        inline=inline,
-        constexpr=constexpr,
-        attrs=attrs,
-        macro_guard=macro_guard,
-        impl_loc=impl_loc,
-        impl_file_suffix=impl_file_suffix,
-    )
+    meta = CudaConstructorMeta(inline=inline,
+                               constexpr=constexpr,
+                               attrs=attrs,
+                               macro_guard=macro_guard,
+                               impl_loc=impl_loc,
+                               impl_file_suffix=impl_file_suffix,
+                               header_only=header_only)
     return markers.meta_decorator(func, meta)
